import QUB
from Buttons import *
from qubtree import Node
import math
import win32com.client
from pythoncom import CoInitialize, CoUninitialize

### -------------------------------------------------------------------------
### Find minima and use to set baseline 
def CSBL(props):
    iContext = props['context']
    iLinePoints = props['linepoints']
    aMinima = []

    ### Get The Data ###
    data=QUB.getChosenData()
    seg = data.find('DataSet').find('Segment')
    QUB.fillData(seg)
    chan=seg.find('Channel')
    iPoints = seg.find('Channel').data.count
    nScaling = data.find('DataSet').find('Scaling').data[0]

    ### Get the baseline object ###
    iChannel = QUB.DataView.currData.activeChannel
    bl=QUB.DataView.currData.baseline(iChannel)
    bl.clearNodes(0,iPoints)

    ### Scan for local minimums and maximums
    iMinPos=0
    iMinVal=chan.data[0]
    for iPoint in range(iPoints):
        if (iPoint > iMinPos + iContext) or (iPoint==iPoints-1):
            aMinima.append( iMinPos )
            iMinPos=iPoint
            iMinVal=chan.data[iPoint]
        if chan.data[iPoint]<iMinVal :
            iMinPos=iPoint
            iMinVal=chan.data[iPoint]

    ### Process each interval
    for iMinima in range(len(aMinima)-1) :
        iStart=aMinima[iMinima]
        iStop=aMinima[iMinima+1]

        ### Slope vars ###
        nSumX=0
        nSumY=0
        nSumXY=0
        nSumXX=0
        
        ### Maxima vars ###
        iMaxPos=iStart
        iMaxVal=chan.data[iStart]
        for iPoint in range(iStart,iStop):
            ### Find slope and intercept of Least Squares Fit of first <n> points
            if iPoint < iStart+iLinePoints :
                nSumX = nSumX+iPoint
                nSumY = nSumY+chan.data[iPoint]
                nSumXY = nSumXY+iPoint*chan.data[iPoint]
                nSumXX = nSumXX+iPoint*iPoint
            
            ### Find maxima between minima ###
            if chan.data[iPoint]>iMaxVal :
                iMaxPos = iPoint
                iMaxVal = chan.data[iPoint]
        
        ### Find slope and intercept of fit line
        nY = nSumXX*iLinePoints - nSumX*nSumX
        nSlope = (nSumXY*iLinePoints - nSumX*nSumY)/nY
        nIntercept = (nSumXX*nSumY-nSumX*nSumXY)/nY
        
        ### Add baseline nodes
        nY=nSlope*iStart+nIntercept
        QUB.Report( str(iStart)+' '+str(round(nY*-1.0*nScaling)) )
        bl.addNode( iStart, round(nY*-1.0*nScaling) )
        nY=nSlope*iMaxPos+nIntercept
        QUB.Report( str(iMaxPos)+' '+str(round(nY*-1.0*nScaling)) )
        bl.addNode( iMaxPos, round(nY*-1.0*nScaling) )

    ### Unload the Data ###
    seg.remove(chan)
    chan=0
    seg=0
    data=0

AddTaskButton( 'Chiara Baseline', # button text
               CSBL,   # function
               'CSBL',       # short text for small buttons
               0,            # show settings dialog on left-click?
               1,            # show settings dialog on right-click?
               "Find local minima and maxima, then set baseline",   # tool-tip
               "Find local minima and maxima, then set baseline",   # description in "Edit Buttons"
               "Chiara Baseline properties:", # caption for the settings dialog
               [TextField("context", "Window size for minima determination :", "4000", acceptIntGreaterThan(1)),
                TextField("linepoints", "Points for line fit :", "1000", acceptIntGreaterThan(1))],
               'Preprocessing' )

### -------------------------------------------------------------------------
### Divide the data into <PtsPer> point sections, find the maximum for each section,
### Find the first point where data exceeds <Frac0> of the peak and <Frac1> of the peak
### then pair the sections and find the difference in the peaks.
###        
def CSPK(props):
    iPtsPer = props['ptsper']
    nFrac0 = props['frac0']
    nFrac1 = props['frac1']
    aPeaks = []

    ### Get The Data ###
    data=QUB.getChosenData()
    seg = data.find('DataSet').find('Segment')
    QUB.fillData(seg)
    chan=seg.find('Channel')
    iPoints = seg.find('Channel').data.count
    nScaling = data.find('DataSet').find('Scaling').data[0]

    ### Get the baseline object ###
    iChannel = QUB.DataView.currData.activeChannel
    bl=QUB.DataView.currData.baseline(iChannel)
    bl.clearNodes(0,iPoints)

    ### Initialize segments [maximum_pos, Maximum_Value, slopebegin, slopeend]
    for iPoint in range(1+(iPoints-1)/iPtsPer):
        aPeaks.append( [iPoint*iPtsPer, chan.data[iPoint*iPtsPer], 0, 0] )
        
    ### Scan for local maximum
    for iPoint in range(iPoints):
        iSeg = iPoint / iPtsPer
        if chan.data[iPoint] > aPeaks[iSeg][1] :
            aPeaks[iSeg] = [iPoint, chan.data[iPoint], 0, 0] 

    ### scan for start and end of slope determination section
    for iPoint in range(iPoints):
        iSeg = iPoint / iPtsPer
        if aPeaks[iSeg][2]==0 :
            if chan.data[iPoint] >= nFrac0*aPeaks[iSeg][1] :
                aPeaks[iSeg][2]=iPoint
        if aPeaks[iSeg][3]==0 :
            if chan.data[iPoint] >= nFrac1*aPeaks[iSeg][1] :
                aPeaks[iSeg][3]=iPoint

    ### Output data
    #for i in range(0, (iPoints-1)/iPtsPer-1, 2) :
    #    QUB.Report('%d %f %d %f' % (aPeaks[i][0],aPeaks[i][1], aPeaks[i+1][0],aPeaks[i+1][1]))

    ### Output data to excel
    excel = win32com.client.Dispatch("Excel.Application")
    excel.Visible = 1
    book = excel.Workbooks.Add(-4167) # XLWBatWorksheet
    sheet = book.Worksheets.Item(1)
    
    cells = sheet.Cells
    iMax=0
    for i in range(0, (iPoints-1)/iPtsPer-1, 2) :
        cells(i/2+3,2).Value = aPeaks[i][0]
        cells(i/2+3,3).Value = aPeaks[i][1]
        cells(i/2+3,4).Value = aPeaks[i+1][0]
        cells(i/2+3,5).Value = aPeaks[i+1][1]
        cells(i/2+3,6).Value = aPeaks[i+1][1]-aPeaks[i][1]
        iMax=i/2+3
        
    sheet.ChartObjects().Add(300,25,400,300)
    chart = sheet.ChartObjects(1).Chart
    chart.ChartType = 65 # xlLineMarkers  (xlLine=4) (xlLineStacked=63)
    chart.SeriesCollection().Add( sheet.Range( sheet.Cells(3,6), sheet.Cells(iMax,6) ) )
    chart.SeriesCollection(1).Border.Color = 0x999999
    chart.SeriesCollection(1).Border.Weight = 0x2 # xlThin    

    excel.Visible = 1

    ### Unload data 
    seg.remove(chan)


AddTaskButton( 'Chiara Peak Diff', # button text
               CSPK,   # function
               'CSPK',       # short text for small buttons
               0,            # show settings dialog on left-click?
               1,            # show settings dialog on right-click?
               "Find differences between successive peaks",   # tool-tip
               "Find differences between successive peaks",   # description in "Edit Buttons"
               "Chiara Peak Difference properties:", # caption for the settings dialog
               [TextField("ptsper", "Points per section :", "400", acceptIntGreaterThan(1)),
                TextField("frac0",  "Slope begins at fraction of peak :", ".1", acceptFloat),
                TextField("frac1",  "Slope ends at fraction of peak :", ".5", acceptFloat)],
               'Preprocessing' )

### -------------------------------------------------------------------------
### Find Groups of 4 peaks and Excel output the location and magnitude
def CSPP(props):
    iPtsPer = props['ptsper']
    aPeaks = []

    ### Get The Data ###
    iSum=0
    data=QUB.getChosenData()
    nScaling = data.find('DataSet').find('Scaling').data[0]
    iChannel = QUB.DataView.currData.activeChannel
    seg = data.find('DataSet').find('Segment')
    while not seg.isNull:
        QUB.fillData(seg)
        chan=seg.find('Channel')
        iPoints = seg.find('Channel').data.count
        ### initialize
        if iSum==0 :
            iMinVal=chan.data[0]
            iMinPos=0
            iMaxVal=chan.data[0]
            iMaxPos=0
            
        ### Scan for local maximums/minimums
        for iPoint in range(iPoints):
            # set minimum
            if iMinVal >= chan.data[iPoint] :
                iMinVal = chan.data[iPoint]
                iMinPos = iSum+iPoint
            # set maximum
            if iMaxVal <= chan.data[iPoint] :
                iMaxVal = chan.data[iPoint]
                iMaxPos = iSum+iPoint
            # if no new max / min in last <n> points
            if (iSum+iPoint > iMinPos+iPtsPer) and (iSum+iPoint > iMaxPos+iPtsPer) :
                if iMaxPos > iMinPos :
                    aPeaks.append( [iMaxPos, iMaxVal] )
                iMinVal=chan.data[iPoint]
                iMinPos=iSum+iPoint
                iMaxVal=chan.data[iPoint]
                iMaxPos=iSum+iPoint
        
        iSum = iSum + iPoints
        seg = seg.next('Segment')

    ### Output data to excel - 1st sheet - sets of 4
    excel = win32com.client.Dispatch("Excel.Application")
    excel.Visible = 1
    book = excel.Workbooks.Add(-4167) # XLWBatWorksheet
    sheet = book.Worksheets.Item(1)
    cells = sheet.Cells
    
    cells(22,2).Value = "Peak 1"
    cells(22,4).Value = "Peak 2"
    cells(22,6).Value = "Peak 3"
    cells(22,8).Value = "Peak 4"
    cells(22,10).Value = "Peak4-Peak2"
    iMax=0
    for i in range(0, len(aPeaks)-3, 4) :
        cells(i/4+23,2).Value = aPeaks[i][0]
        cells(i/4+23,3).Value = aPeaks[i][1]
        cells(i/4+23,4).Value = aPeaks[i+1][0]
        cells(i/4+23,5).Value = aPeaks[i+1][1]
        cells(i/4+23,6).Value = aPeaks[i+2][0]
        cells(i/4+23,7).Value = aPeaks[i+2][1]
        cells(i/4+23,8).Value = aPeaks[i+3][0]
        cells(i/4+23,9).Value = aPeaks[i+3][1]
        cells(i/4+23,10).Value = aPeaks[i+3][1]-aPeaks[i+1][1]
        iMax=i/4+23
        
    sheet.ChartObjects().Add(1,1,500,250)
    chart = sheet.ChartObjects(1).Chart
    chart.ChartType = 65 # xlLineMarkers  (xlLine=4) (xlLineStacked=63)
    Range = sheet.Range( sheet.Cells(23,10), sheet.Cells(int(iMax),10) )

    chart.SeriesCollection().NewSeries()
    chart.SeriesCollection(1).Values = Range
    #chart.SeriesCollection().Add( Range )
    chart.SeriesCollection(1).Border.Color = 0x999999
    chart.SeriesCollection(1).Border.Weight = 0x2 # xlThin    

    ### Output data to excel - 2nd sheet - sets of 2
    book.Worksheets.Add()
    sheet = book.Worksheets.Item(1)
    cells = sheet.Cells
    
    cells(22,2).Value = "Peak 1"
    cells(22,4).Value = "Peak 2"
    cells(22,6).Value = "Peak2-Peak1"
    iMax=0
    for i in range(0, len(aPeaks)-1, 2) :
        cells(i/2+23,2).Value = aPeaks[i][0]
        cells(i/2+23,3).Value = aPeaks[i][1]
        cells(i/2+23,4).Value = aPeaks[i+1][0]
        cells(i/2+23,5).Value = aPeaks[i+1][1]
        cells(i/2+23,6).Value = aPeaks[i+1][1]-aPeaks[i][1]
        iMax=i/2+23
        
    sheet.ChartObjects().Add(1,1,500,250)
    chart = sheet.ChartObjects(1).Chart
    chart.ChartType = 65 # xlLineMarkers  (xlLine=4) (xlLineStacked=63)
    Range = sheet.Range( sheet.Cells(23,6), sheet.Cells(int(iMax),6) )
    chart.SeriesCollection().NewSeries()
    chart.SeriesCollection(1).Values = Range
    #chart.SeriesCollection().Add( Range )
    chart.SeriesCollection(1).Border.Color = 0x999999
    chart.SeriesCollection(1).Border.Weight = 0x2 # xlThin    
    excel.Visible = 1

    ### Unload the Data ###
    seg.remove(chan)
    chan=0
    seg=0
    data=0

AddTaskButton( 'Chiara Piezo Peaks', # button text
               CSPP,   # function
               'CSPP',       # short text for small buttons
               0,            # show settings dialog on left-click?
               1,            # show settings dialog on right-click?
               "Find Piezo Peaks",   # tool-tip
               "Find Piezo Peaks",   # description in "Edit Buttons"
               "Piezo Peaks:", # caption for the settings dialog
               [TextField("ptsper", "Peak = no higher point within ... (points):", "50", acceptIntGreaterThan(1))],
               'Preprocessing' )

### -------------------------------------------------------------------------
### Find peak per segment, Excel output the time and rise for the deflection.
def PP2(props):
    iChnDef=props['ChnlDeflect']
    iChnPzo=props['ChnlPiezo']
    iStartSeg=props['StartSegment']
    nPrePeak=props['PrePeak']
    cChartFunc=props['ChartFunc']

    aPeaks = []

    ### Get The Data - dataDef and dataChn
    alldata = QUB.DataView.currData
    iSaveChannel=alldata.activeChannel
    alldata.activeChannel=iChnDef
    dataDef=QUB.getChosenData()
    alldata.activeChannel=iChnPzo
    dataPzo=QUB.getChosenData()
    alldata.activeChannel=iSaveChannel
    
    ### scan the data for peaks 
    iSeg = 0
    iSum = 0              # total points scanned
    segDef = dataDef.find('DataSet').find('Segment')
    sampling=dataDef.find('DataSet')['sampling'].data[0]/1000
    segPzo = dataPzo.find('DataSet').find('Segment')
    pkList = []
    while not segDef.isNull :
        QUB.fillData(segDef)
        chanDef = segDef.find('Channel')
        iPointsDef = chanDef.data.count

        QUB.fillData(segPzo)
        chanPzo = segPzo.find('Channel')
        iPointsPzo = chanPzo.data.count
        
        if iStartSeg==iSeg : 
            ### initialize
            iMaxVal = chanDef.data[0]
            iMaxPos = iSum
                
            ### Scan segment for maximum
            for iPoint in range(iPointsDef):
                if iMaxVal < chanDef.data[iPoint] :
                    iMaxVal = chanDef.data[iPoint]
                    iMaxPos = iSum+iPoint

            if iMaxPos-iSum > 10 : 
                ###### Least squares fit for baseline.  (This should be a separate function.)
                ### Slope vars ###
                nSumX=0
                nSumY=0
                nSumXY=0
                nSumXX=0
                ### Maxima vars ###
                iLinePoints = int(nPrePeak*(iMaxPos-iSum))
                for iPoint in range(0,iLinePoints):
                    ### Find slope and intercept of Least Squares Fit of first <n> points
                    nSumX = nSumX+iPoint
                    nSumY = nSumY+chanDef.data[iPoint]
                    nSumXY = nSumXY+iPoint*chanDef.data[iPoint]
                    nSumXX = nSumXX+iPoint*iPoint
                ### Find slope and intercept of fit line
                nY = nSumXX*iLinePoints - nSumX*nSumX
                nSlope = (nSumXY*iLinePoints - nSumX*nSumY)/nY
                nIntercept = (nSumXX*nSumY-nSumX*nSumXY)/nY

                ### Find first point prior to peak below fit line
                iPoint = iMaxPos-iSum
                while iPoint>0 and nSlope*(iPoint)+nIntercept<chanDef.data[iPoint] :
                    iPoint = iPoint - 1
                iMinPos = iPoint+iSum
                iMinVal = chanDef.data[iPoint]

                ### Add to array
                if (iMinPos-iSum) < iPointsPzo :
                    iPiezoMin = chanPzo.data[iMinPos-iSum]
                else :
                    iPiezoMin = 0
                if (iMaxPos-iSum) < iPointsPzo :
                    iPiezoMax = chanPzo.data[iMaxPos-iSum]
                else :
                    iPiezoMax = 0
                aPeaks.append( [ iSeg, iMinPos, iMinVal, iPiezoMin, iMaxPos, iMaxVal, iPiezoMax ] )

                ### Add to selection list 
                pkList.append( (iMinPos, iMaxPos) )

            ### Set next seg to analyze - skip every other segment
            iStartSeg = iStartSeg+2       
            
        segDef = segDef.next('Segment')
        segPzo = segPzo.next('Segment')
        iSum = iSum + iPointsDef
        iSeg = iSeg+1

    ### Create selection list 
    pkList.append( (iMinPos, iMaxPos) )
    alldata.setList('PP2', pkList)
    
    ### Output data to excel - 1st sheet - sets of 4
    excel = win32com.client.Dispatch("Excel.Application")
    excel.Visible = 1
    book = excel.Workbooks.Add(-4167) # XLWBatWorksheet
    sheet = book.Worksheets.Item(1)
    cells = sheet.Cells
    
    cells(22,1).Value = "Segment"
    cells(22,2).Value = "Min Pos"
    cells(22,3).Value = "Deflection"
    cells(22,4).Value = "Piezo"
    cells(22,5).Value = "Max Pos"
    cells(22,6).Value = "Deflection"
    cells(22,7).Value = "Piezo"
    cells(22,8).Value = "Piezo Delta"
    cells(22,9).Value = cChartFunc

    for i in range(0, len(aPeaks)-1) :
        cells(23+i,1).Value = aPeaks[i][0]
        cells(23+i,2).Value = sampling*aPeaks[i][1]
        cells(23+i,3).Value = aPeaks[i][2]
        cells(23+i,4).Value = aPeaks[i][3]
        cells(23+i,5).Value = sampling*aPeaks[i][4]
        cells(23+i,6).Value = aPeaks[i][5]
        cells(23+i,7).Value = aPeaks[i][6]
        cells(23+i,8).Value = aPeaks[i][6]-aPeaks[i][3]
        # user function f(t1,t0,y1,y0,p1,p0) - Convert to f(cell references)
        cFunc = cChartFunc.replace('t0','B'+str(23+i))
        cFunc = cFunc.replace('t1','E'+str(23+i))
        cFunc = cFunc.replace('y0','C'+str(23+i))
        cFunc = cFunc.replace('y1','F'+str(23+i))
        cFunc = cFunc.replace('p0','D'+str(23+i))
        cFunc = cFunc.replace('p1','G'+str(23+i))
        cells(23+i,9).Formula = '='+cFunc
        
    sheet.ChartObjects().Add(1,1,500,250)
    chart = sheet.ChartObjects(1).Chart
    chart.ChartType = 65 # xlLineMarkers  (xlLine=4) (xlLineStacked=63)

    # plot user function 
    Range = sheet.Range( sheet.Cells(23,9), sheet.Cells(int(22+len(aPeaks)),9) )
    chart.SeriesCollection().NewSeries()
    chart.SeriesCollection(1).Values = Range
    chart.SeriesCollection(1).Border.Color = 0x999999
    chart.SeriesCollection(1).Border.Weight = 0x2 # xlThin    

    excel.Visible = 1

    ### Unload the Data ###
    segDef.remove(chanDef)
    segPzo.remove(chanPzo)

AddTaskButton( 'PiezoPk 2', # button text
               PP2,   # function
               'PP2',       # short text for small buttons
               0,            # show settings dialog on left-click?
               1,            # show settings dialog on right-click?
               "Chart Piezo Peaks",   # tool-tip
               "Chart Piezo Peaks",   # description in "Edit Buttons"
               "Piezo Peaks 2:", # caption for the settings dialog
               [TextField("ChnlDeflect", "Deflection Channel:", "0", acceptIntGreaterThan(-1)),
                TextField("ChnlPiezo", "Piezo Channel:", "1", acceptIntGreaterThan(-1)),
                TextField("StartSegment", "Starting Segment (0 based):", "1", acceptIntGreaterThan(-1)),
                TextField("PrePeak", "Fraction of pre-peak data to establish baseline:", ".7", acceptFloat),
                TextField("ChartFunc", "Function of (t0,t1,y0,y1,p0,p1) to plot:", "(y1-y0)/(t1-t0)")],
               'Preprocessing' )

